{% extends "htmx-theme/templates/base.html" %}

{% block title %}
  {% set html_title = "&lt;/&gt; htmx ~ Examples ~ " ~ page.title %}
{% endblock title %}

{% block content %}
  {% if page.extra and page.extra.show_title is defined %}
    {% set show_title = page.extra.show_title %}
  {% else %}
    {% set show_title = true %}
  {% endif %}
  {% if show_title %}<h1>{{ page.title | safe }}</h1>{% endif %}
  <script src="https://unpkg.com/sinon@9.0.2/pkg/sinon.js"></script>
  <script src="/js/demo.js"></script>
  {{ page.content | safe }}
{% endblock content %}
