{% extends "base.html" %}

{% block title %}
  {% if section.title -%}
    {% set html_title = "&lt;/&gt; htmx ~ " ~ section.title -%}
  {% endif -%}
{% endblock title %}

{% block content %}
  {% if section.extra and section.extra.show_title is defined -%}
    {% set show_title = section.extra.show_title -%}
  {% else -%}
    {% set show_title = true -%}
  {% endif -%}
  {% if show_title %}<h1>{{ section.title | safe }}</h1>{% endif %}
  {{ section.content | safe }}
{% endblock content %}
